import { createPersistentStateStore, } from "@reactive-js/ix";
import { concat, ofValue, share } from "@reactive-js/observable";
import { fromEvent } from "./event";
import { pipe } from "@reactive-js/pipe";
const emptyLocation = {
    fragment: "",
    path: "",
    query: "",
};
const locationEquals = (a, b) => a === b ||
    (a.path === b.path && a.query === b.query && a.fragment === b.fragment);
const chopLeadingChar = (str) => str.length > 0 ? str.substring(1) : "";
const getCurrentLocation = (_) => {
    const path = window.location.pathname;
    const query = chopLeadingChar(window.location.search);
    const fragment = chopLeadingChar(window.location.hash);
    return { path, query, fragment };
};
const dispatch = (newLocation) => {
    const currentLocation = getCurrentLocation();
    if (!locationEquals(currentLocation, newLocation)) {
        const { path, query, fragment } = newLocation;
        let uriString = path;
        uriString = query.length > 0 ? `${uriString}?${query}` : uriString;
        uriString = fragment.length > 0 ? `${uriString}#${fragment}` : uriString;
        window.history.pushState(undefined, "", uriString);
    }
};
export const createLocationStoreResource = (scheduler) => {
    const observable = pipe(concat(ofValue(getCurrentLocation()), fromEvent(window, "popstate", getCurrentLocation)), share(scheduler));
    const historyIterator = {
        get subscriberCount() {
            return observable.subscriberCount;
        },
        dispatch,
        subscribe: subscriber => observable.subscribe(subscriber),
    };
    return createPersistentStateStore(historyIterator, emptyLocation, scheduler, locationEquals);
};
//# sourceMappingURL=locationStore.js.map